/* MJG 27/12/94
		pl_read()
*/

#include "isomanifest.h"
#include "namelist.h"
#include "demanifest.h"
#include "util.h"
#include "destrings.h"
#ifdef DOS
#include <stdlib.h>
#include "proto-de.h"
#endif DOS
#include "messages.h"
#include <stdio.h>
#include <ctype.h>
#include <pwd.h>
#include <signal.h>
#include <setjmp.h>
#ifdef DOS
#include <msdos.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#endif DOS
#ifdef USELDAP
#include <string.h>
#ifdef IC
#include <ldap/lber.h>
#include <ldap/ldap.h>
#else
#include <lber.h>
#include <ldap.h>
#endif /* IC */
#else 
#include "quipu/list.h"
#include "quipu/attrvalue.h"
#include "quipu/ds_search.h"
#include "quipu/connection.h"
#include "quipu/dua.h"
#endif USELDAP


#ifdef USELDAP
extern LDAP * ldap_ld;
extern struct namelist * platts;
#else
extern Attr_Sequence plas;
extern int abandoned;
#endif USELDAP


static unsigned char escapes[16][25] = {
	'A','C','E','I','N','O','U','Y','L','S','X','Z', 
	'a','c','e','i','n','o','u','y', 'l', 's', 'x', 'z', ' ',

/* Grave upper case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',
/* Grave lower case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '`',

/* Acute upper case */
	0xc1, 0xc6, 0xc9, 0xcd, 0xd1, 0xd3, 0xda, 0xdd, '?', 0xa6, '?', 0xac,
/* Acute lower case */
	0xe1, 0xe6, 0xe9, 0xed, 0xf1, 0xf3, 0xfa, 0xfd, '?', 0xb6, '?', 0xbc, '\'',

/* Circumflex upper case */
        0xc2, '?', 0xca, 0xce, '?', 0xd4, 0xdb, '?', '?', '?', '?', '?',
/* Circumflex lower case */
	0xe2, '?', 0xea, 0xee, '?', 0xf4, 0xfb, '?', '?', '?', '?', '?', '?', 

/* Tilde upper case */
	0xc3, '?', 0xcb, 0xcf, 0xd1, 0xd5, 0xdc, '?', '?', '?', '?', '?', 
/* Tilde lower case */
	0xe3, '?', 0xeb, 0xef, 0xf1, 0xf5, 0xfc, '?', '?', '?', '?', '?', '?',

/* Macron upper case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', 
/* Macron lower case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', 

/* ? upper case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',
/* ? lower case */
	'?', '?', '?', '?', '?', '?', '?', '?',  '?', '?', '?', '?', '?',

/* Dot upper case */
	'?', '?', '?', '?', '?', 0xd3, '?', '?', '?', '?', '?', 0xaf,
/* Dot lower case */
	'?', '?', '?', '?', '?', 0xf3, '?', '?', '?', '?', '?', 0xbf, '?',

/* Diaresis upper case */
        0xc4, '?', 0xcb, '?', '?', 0xd6, 0xdc, 0x82, '?', '?', '?', '?',
/* Diaresis lower case */
        0xe4, '?', 0xeb, '?', '?', 0xf6, 0xfc, 0xff, '?', '?', '?', '?', '"',

/* Umlaut upper case */
        0xc4, '?', 0xcb, 0xcf, '?', 0xd6, 0xdc, 0x82, '?', '?', '?', '?',
/* Umlaut lower case */
        0xe4, '?', 0xeb, 0xef, '?', 0xf6, 0xfc, 0xff, '?', '?', '?', '?', '"',

/* Ring upper case */
	0xc5, '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', 
/* Ring lower case */
	0xe5, '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',

/* Cedilla upper case */
	'?', 0xc7, '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',
/* Cedilla lower case */
	'?', 0xe7, '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',

/* Underline upper case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', 
/* Underline lower case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '_',


/* Umlaut upper case */
        0xc4, '?', 0xcb, 0xcf, '?', 0xd6, 0xdc, 0x82, '?', '?', '?', '?',
/* Umlaut lower case */
        0xe4, '?', 0xeb, 0xef, '?', 0xf6, 0xfc, 0xff, '?', '?', '?', '?', '"',

/* Ogonek upper case */
	0xa1, '?', 0xca, '?', '?', '?', '?', '?', '?', '?', '?', '?',
/* Ogonek lower case */
	0xb1, '?', 0xea, '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',

/* ? upper case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', 
/* ? lower case */
	'?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', '?',
};

static unsigned char trans[32] = {
	'?', '?', 0xd0, '?', '?', '?', '?', 'L', 
	0xa3, '?', '?', '?', '?', 'T', 'N', 'n',
	'K', '?', 'd', '?', 'h', 'i', '?', 'l',
	0xb3, '?', '?', '?', '?', 't', 'N', '?'};

static unsigned char trans1[32] = {
	'?', '?', 0xd0, '?', '?', '?', '?', '?', 
	0xa3, '?', '?', '?', '?', '?', '?', '?',
	'?', '?', '?', '?', '?', '?', '?', '?',
	0xb3, '?', '?', '?', '?', '?', '?', '?'};
char * toiso8859(sstr)
char *sstr;
{
unsigned char *str;
int n;
static unsigned char *buff = (unsigned char *)0;
unsigned char *optr;

  if (buff == (unsigned char *)0)
	  buff = (unsigned char *) smalloc (BUFSIZ);

  optr = buff;

  str = (unsigned char *) sstr;
  while (*str != '\0')
  {
  if ((*str > 0x1f) && (*str < 0x80)) {
	*optr++ = *str++;}
  else if ((*str>0xa0) && (*str < 0xc0)) { 
	*optr++ = *str++;}
  else if (*str > 0xdf) {
	*optr++ = trans[((int)(*str++)) - 0xe0];}
  else if ((*str > 0xbf) && (*str < 0xd0)) {
	n = (int) (*str - (unsigned char) 0xc0);
	str++;
	if (*str == '\0')
		return;
	switch (*str) {
	   case 'A':
		*optr++ = escapes[n][0];
		break;
	   case 'C':
		*optr++ = escapes[n][1];
		break;
	   case 'E':
		*optr++ = escapes[n][2];
		break;
	   case 'I':
		*optr++ = escapes[n][3];
		break;
	   case 'N':
		*optr++ = escapes[n][4];
		break;
	   case 'O':
		*optr++ = escapes[n][5];
		break;
	   case 'U':
		*optr++ = escapes[n][6];
		break;
	   case 'Y':
		*optr++ = escapes[n][7];
		break;
	   case 'L':
		*optr++ = escapes[n][8];
		break;
	   case 'S':
		*optr++ = escapes[n][9];
		break;
	   case 'X':
		*optr++ = escapes[n][10];
		break;
	   case 'Z':
		*optr++ = escapes[n][11];
		break;
	   case 'a':
		*optr++ = escapes[n][12];
		break;
	   case 'c':
		*optr++ = escapes[n][13];
		break;
	   case 'e':
		*optr++ = escapes[n][14];
		break;
	   case 'i':
		*optr++ = escapes[n][15];
		break;
	   case 'n':
		*optr++ = escapes[n][16];
		break;
	   case 'o':
		*optr++ = escapes[n][17];
		break;
	   case 'u':
		*optr++ = escapes[n][18];
		break;
	   case 'y':
		*optr++ = escapes[n][19];
		break;
	   case 'l':
		*optr++ = escapes[n][20];
		break;
	   case 's':
		*optr++ = escapes[n][21];
		break;
	   case 'x':
		*optr++ = escapes[n][22];
		break;
	   case 'z':
		*optr++ = escapes[n][23];
		break;
	   case ' ':
		*optr++ = escapes[n][24];
		break;
	   default:
		*optr++ = '?';
	 };
	str++;
	}
	else str++;
  };
  *optr++ = NULL;
  return (buff);
}
/* MJG 23/07/94 */
unsigned char * toT61(sstr)
unsigned char *sstr;
{
 unsigned char *str;
 static unsigned char *buff = (unsigned char *)0;
 unsigned char *optr;
 int i,j,fin;
 if (buff == (unsigned char *)0)
	buff = (unsigned char *) smalloc (BUFSIZ);
 optr = buff;
 str = (unsigned char *) sstr;
 while (*str != '\0')
 {
	fin = 0;
	for (i=1; i<16; i++)
	{
	 for (j=0; j<24; j++)
		if (escapes[i][j] == *str)
		{
#ifdef USELDAP
			(void) sprintf (optr,"\\\\%02x", 0xc0 | (i & 0xf));
			optr+= 4; 
#else
			(void) sprintf (optr,"\\%02x", 0xc0 | (i & 0xf));
			optr+= 3; 
#endif
			*optr++= escapes[0][j];
			*str++;
			fin=1;
			break;
		}
	 if (fin) break;
        }
    	for (i=1; i<32; i++)
		if (trans1[i] == *str)
		{
#ifdef USELDAP
			(void) sprintf (optr,"\\\\%02x", 0xe0 | (i & 0x1f));
			optr+= 4; 
#else
			(void) sprintf (optr,"\\%02x", 0xe0 | (i & 0x1f));
			optr+= 3; 
#endif
			*str++;

			fin=1;
			break;
		}
	if (fin) continue;
	*optr++= *str++;
 }
 *optr++= NULL;
 return buff;
} 

char * unquotechar (a,b)
char *a, *b;
{
 int val;
 switch (*a) {
        case '\\':
                *b = '\\';
                break;
        case 'n':
            *b = '\n';
            break;
        case 't':
            *b = '\t';
        case 'r':
            *b = '\r';
            break;
        default:
            if (isxdigit((*a) & 0xff) && sscanf (a,"%2x", &val) == 1) {
                *b = val & 0xff;
                a++;
            }
            break;
 
        }
 return (a);
}

char * t61parse(str)
register char * str;
{
 char buffer [BUFSIZ];
 register char * ptr=buffer;
 register int i;
 for (i=0; *str != 0; str++,i++) {
	if (*str != '\\')
		*ptr++ = *str;
	else {
		str++, i++;
		str = (char *)unquotechar (str,ptr);
		ptr++;
              }
		
	}
 *ptr = 0;
 if (index(buffer, '\\') != NULLCP)
	(void) strcpy(buffer,t61parse(buffer));
 return (strdup(buffer));
}

#ifdef USELDAP

pl_init()
{
  if (platts)
	return TRUE;
  addToList(&platts, DE_OBJECT_CLASS);
  addToList(&platts, DE_PL_RDN);
  return TRUE;
}
#else
Attr_Sequence plas = NULL;
AttributeType at_ObjectClass;
AttributeType at_Plrdn;
pl_init()
{
 Attr_Sequence as;
 int result=TRUE;
 if (plas)
	return result;
 if ( (at_ObjectClass=AttrT_new("ObjectClass")) == NULLAttrT) 
	result= FALSE;
 if ( (at_ObjectClass=AttrT_new("Plrdn")) == NULLAttrT) 
	result= FALSE;
 plas= as_comp_new(at_ObjectClass, NULLAV, NULLACL_INFO);
 plas= as_merge(plas, as_comp_new(at_ObjectClass, NULLAV, NULLACL_INFO));
}
#endif




int pl_read (char * str, char * pl_str)
{
 static int plinited = FALSE;
#ifdef USELDAP
 
 BerElement *cookie;
 char * at;
 char ** av;
 char * cp;
 char ** atpp;
 LDAPMessage * entry;
#else
 static struct ds_read_arg rarg;
 static struct ds_read_result rresult;
 static struct DSError rerror;
 static CommonArgs rca = default_common_args;
 Attr_Sequence * atl;
 Attr_Sequence  as;
 AttributeType at;
#endif USELDAP
 int i, retval, pl_obj;
 

#ifdef USELDAP
 if ( (!platts) && !(plinited = pl_init()))
	return plinited;
 atpp = makeAllAttrs(platts);
 retval= ldap_search_s(ldap_ld, str, LDAP_SCOPE_BASE, "objectClass=*", atpp, 0, &entry);
 free((char **)atpp);
 if (retval != LDAP_SUCCESS)
 { 
  ldap_perror(ldap_ld, "pl_read");
  return 0;
 }
 for (at = ldap_first_attribute(ldap_ld, entry, &cookie);
	at != NULL; at = ldap_next_attribute(ldap_ld, entry, cookie))
 {
	pl_obj= 0;
	if ( !strcmp(at, DE_OBJECT_CLASS) )
	{
 		av= ldap_get_values(ldap_ld, entry, at);
		for (i=0; av[i] != NULL; i++)
		{
		  if (!strcasecmp(av[i], DE_PL_OBJECT))
		  {
			pl_obj= 1;
			break;			
		  }	
		}
		ldap_value_free(av);
		if (!pl_obj)
			return 0;
	} else
	  if ( !strcmp(at, DE_PL_RDN) )
	  {
		av= ldap_get_values(ldap_ld, entry, at);
		(void) strcpy(pl_str, "");
		if ( index(av[0], ',') != NULLCP )
			strcat(pl_str, "\"");
		strcat(pl_str, av[0]);          /* wartosc atrybutu */
		if ( index(av[0], ',') != NULLCP )
			strcat(pl_str, "\"");
		ldap_value_free(av);	
		return 1;
	  }
 }		
 return 0;
#else
 if ( (!plas) && !(plinited = pl_init()))
	return plinited;
 rarg.rda_common = rca; /* struct copy */
 rarg.rda_common.ca_servicecontrol.svc_timelimit = SVC_NOTIMELIMIT;
 rarg.rda_common.ca_servicecontrol.svc_sizelimit = SVC_NOSIZELIMIT;
 rarg.rda_object = str2dn(str);
 /* specify attributes of interest */
 rarg.rda_eis.eis_allattributes = FALSE;
/*
 atl = &(rarg.rda_eis.eis_select);
 for (as = plas; as != NULLATTR; as = as->attr_link)
 {
	if ( as->attr_type == NULLAttrT)
                continue;
        *atl = as_comp_alloc();
        (*atl)->attr_type = as->attr_type;
        (*atl)->attr_value = NULLAV;
        (*atl)->attr_link = as->attr_link;
  }
*/
  rarg.rda_eis.eis_select = plas;
  rarg.rda_eis.eis_infotypes = EIS_ATTRIBUTESANDVALUES;
  retval = de_read(&rarg, &rerror, &rresult, DE_UNSIGNED);
  if ((retval == DSE_INTR_ABANDONED) &&
      (rerror.dse_type == DSE_ABANDONED))
    abandoned = TRUE;
  if (retval != OK)
    return 0;
  if (rresult.rdr_entry.ent_attr == NULL)
    return 0;
  (void) strcpy(pl_str, (char *)val2str(rresult.rdr_entry.ent_attr->attr_value));
  return 1; 
   
#endif USELDAP
}
